@@ -11,10 +11,10 @@ from collections import defaultdict
 from functools import partial
 from qt.core import QApplication, QDialog, QPixmap, QTimer
 
-from calibre import as_unicode, guess_type
+from calibre import as_unicode, guess_type, prepare_string_for_xml
 from calibre.constants import iswindows
 from calibre.ebooks import BOOK_EXTENSIONS
-from calibre.ebooks.metadata import MetaInformation
+from calibre.ebooks.metadata import MetaInformation, normalize_isbn
 from calibre.gui2 import (
     choose_dir, choose_files, error_dialog, gprefs, info_dialog, question_dialog,
     warning_dialog
@@ -59,7 +59,7 @@ class AddAction(InterfaceAction):
         self._add_filesystem_book = self.Dispatcher(self.__add_filesystem_book)
         self.add_menu = self.qaction.menu()
         ma = partial(self.create_menu_action, self.add_menu)
-        ma('recursive-add', _('Add from directories and sub-directories')).triggered.connect(self.add_recursive_question)
+        ma('recursive-add', _('Add from folders and sub-folders')).triggered.connect(self.add_recursive_question)
         ma('archive-add-book', _('Add multiple books from archive (ZIP/RAR)')).triggered.connect(self.add_from_archive)
         self.add_menu.addSeparator()
         ma('add-empty', _('Add empty book (Book entry with no formats)'),
@@ -363,8 +363,35 @@ class AddAction(InterfaceAction):
             for path in temp_files:
                 os.remove(path)
 
-    def add_isbns(self, books, add_tags=[]):
-        self.isbn_books = list(books)
+    def check_for_existing_isbns(self, books):
+        db = self.gui.current_db.new_api
+        book_id_identifiers = db.all_field_for('identifiers', db.all_book_ids(tuple))
+        existing_isbns = {normalize_isbn(ids.get('isbn', '')): book_id for book_id, ids in book_id_identifiers.items()}
+        existing_isbns.pop('', None)
+        ok = []
+        duplicates = []
+        for book in books:
+            q = normalize_isbn(book['isbn'])
+            if q and q in existing_isbns:
+                duplicates.append((book, existing_isbns[q]))
+            else:
+                ok.append(book)
+        if duplicates:
+            det_msg = '\n'.join(f'{book["isbn"]}: {db.field_for("title", book_id)}' for book, book_id in duplicates)
+            if question_dialog(self.gui, _('Duplicates found'), _(
+                'Books with some of the specified ISBNs already exist in the calibre library.'
+                ' Click "Show details" for the full list. Do you want to add them anyway?'), det_msg=det_msg
+            ):
+                ok += [x[0] for x in duplicates]
+        return ok
+
+    def add_isbns(self, books, add_tags=[], check_for_existing=False):
+        books = list(books)
+        if check_for_existing:
+            books = self.check_for_existing_isbns(books)
+            if not books:
+                return
+        self.isbn_books = books
         self.add_by_isbn_ids = set()
         self.isbn_add_tags = add_tags
         QTimer.singleShot(10, self.do_one_isbn_add)
@@ -490,7 +517,7 @@ class AddAction(InterfaceAction):
         from calibre.gui2.dialogs.add_from_isbn import AddFromISBN
         d = AddFromISBN(self.gui)
         if d.exec_() == QDialog.DialogCode.Accepted and d.books:
-            self.add_isbns(d.books, add_tags=d.set_tags)
+            self.add_isbns(d.books, add_tags=d.set_tags, check_for_existing=d.check_for_existing)
 
     def add_books(self, *args):
         '''
@@ -547,10 +574,10 @@ class AddAction(InterfaceAction):
                 merged[author].append(title)
             lines = []
             for author in sorted(merged, key=sort_key):
-                lines.append(author)
-                for title in sorted(merged[author], key=sort_key):
-                    lines.append('\t' + title)
-                lines.append('')
+                lines.append(f'<b><i>{prepare_string_for_xml(author)}</i></b><ol style="margin-top: 0">')
+                for title in sorted(merged[author]):
+                    lines.append(f'<li>{prepare_string_for_xml(title)}</li>')
+                lines.append('</ol>')
             pm = ngettext('The following duplicate book was found.',
                           'The following {} duplicate books were found.',
                           len(adder.merged_books)).format(len(adder.merged_books))
@@ -558,7 +585,7 @@ class AddAction(InterfaceAction):
                 _('Incoming book formats were processed and merged into your '
                     'calibre database according to your auto-merge '
                     'settings. Click "Show details" to see the list of merged books.'),
-                    det_msg='\n'.join(lines), show=True)
+                    det_msg='\n'.join(lines), show=True, only_copy_details=True)
 
         if adder.number_of_books_added > 0 or adder.merged_books:
             # The formats of the current book could have changed if
