@@ -11,7 +11,7 @@ from collections import OrderedDict
 from functools import partial
 from polyglot.builtins import iteritems, itervalues, map, unicode_type
 
-from PyQt5.Qt import (QDialog, QGridLayout, QIcon, QCheckBox, QLabel, QFrame,
+from qt.core import (QDialog, QGridLayout, QIcon, QCheckBox, QLabel, QFrame,
                       QApplication, QDialogButtonBox, Qt, QSize, QSpacerItem,
                       QSizePolicy, QTimer, QModelIndex, QTextEdit,
                       QInputDialog, QMenu)
@@ -130,14 +130,14 @@ class Polish(QDialog):  # {{{
         self.bb = bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok|QDialogButtonBox.StandardButton.Cancel)
         bb.accepted.connect(self.accept)
         bb.rejected.connect(self.reject)
-        self.save_button = sb = bb.addButton(_('&Save settings'), bb.ActionRole)
+        self.save_button = sb = bb.addButton(_('&Save settings'), QDialogButtonBox.ButtonRole.ActionRole)
         sb.clicked.connect(self.save_settings)
-        self.load_button = lb = bb.addButton(_('&Load settings'), bb.ActionRole)
+        self.load_button = lb = bb.addButton(_('&Load settings'), QDialogButtonBox.ButtonRole.ActionRole)
         self.load_menu = QMenu(lb)
         lb.setMenu(self.load_menu)
-        self.all_button = b = bb.addButton(_('Select &all'), bb.ActionRole)
+        self.all_button = b = bb.addButton(_('Select &all'), QDialogButtonBox.ButtonRole.ActionRole)
         connect_lambda(b.clicked, self, lambda self: self.select_all(True))
-        self.none_button = b = bb.addButton(_('Select &none'), bb.ActionRole)
+        self.none_button = b = bb.addButton(_('Select &none'), QDialogButtonBox.ButtonRole.ActionRole)
         connect_lambda(b.clicked, self, lambda self: self.select_all(False))
         l.addWidget(bb, count+1, 1, 1, -1)
         self.setup_load_button()
@@ -334,9 +334,9 @@ class Report(QDialog):  # {{{
         bb = self.bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Close)
         bb.accepted.connect(self.accept)
         bb.rejected.connect(self.reject)
-        b = self.log_button = bb.addButton(_('View full &log'), bb.ActionRole)
+        b = self.log_button = bb.addButton(_('View full &log'), QDialogButtonBox.ButtonRole.ActionRole)
         b.clicked.connect(self.view_log)
-        bb.button(bb.Close).setDefault(True)
+        bb.button(QDialogButtonBox.StandardButton.Close).setDefault(True)
         l.addWidget(bb, 2, 1)
 
         self.finished.connect(self.show_next, type=Qt.ConnectionType.QueuedConnection)
@@ -361,7 +361,7 @@ class Report(QDialog):  # {{{
         self.setWindowTitle(_('Polishing of %s')%book_title)
         self.view.setText(markdown('# %s\n\n'%book_title + report,
                                    output_format='html4'))
-        self.bb.button(self.bb.Close).setFocus(Qt.FocusReason.OtherFocusReason)
+        self.bb.button(QDialogButtonBox.StandardButton.Close).setFocus(Qt.FocusReason.OtherFocusReason)
         self.backup_msg.setVisible(bool(fmts))
         if fmts:
             m = ngettext('The original file has been saved as %s.',
